#pragma once
#include <iostream>
#include <list>
#include "pwt_adjlist.h"

namespace faudes {
	
AdjacencyList::AdjacencyList(Generator g) {

	this->size = g.Size();
	this->adj_list = new std::list<Idx>[size];
	StateSet::Iterator it;
	StateSet successors;
	StateSet::Iterator succ_it;		
	int i = 0;
		
	if (IsConsecutive(g)) {
		
		this->isCons = true;
	}
	else {
		
		this->isCons = false;
	}
		
	if (isCons) { 
		
		for (it = g.StatesBegin(); it != g.StatesEnd(); ++it) {

			successors = g.SuccessorStates(*it);
			
			for (succ_it = successors.Begin(); succ_it != successors.End(); ++succ_it) {
				
				if (*it != *succ_it) {
					
					adj_list[i].push_back(*succ_it);
				}				
			}	
		
			i++;
		}
	}		
	else {
				
		this->idxArray = new Idx[size];
		
		for (it = g.StatesBegin(); it != g.StatesEnd(); ++it) {
	
			this->idxArray[i] = *it;
			i++;		
		}
			
		i = 0;
			
		for (it = g.StatesBegin(); it != g.StatesEnd(); ++it) {

			successors = g.SuccessorStates(*it);
				
			for (succ_it = successors.Begin(); succ_it != successors.End(); ++succ_it) {
				
				if (*it != *succ_it) {
					
					adj_list[i].push_back(FindIdx(*succ_it)+1);
				}				
			}
				
			i++;	
		}		
	}			
}

// This constructor creates the adjacency list from an adjacency matrix,
// which is the method mentioned and Klíma and Polák's article.

AdjacencyList::AdjacencyList(AdjacencyMatrix m) {
	
	this->size = m.height;
	this->adj_list = new std::list<Idx>[size];
	this->isCons = true;
	int i, j, k;
	Idx idx;
	bool * isPresent = new bool[size];
		
	for (i = 0; i < this->size; i++) {
		
		isPresent[i] = false;		
	}
	
	for (i = 0; i < this->size; i++) {
			
		for (j = 0; j < m.width; j++) {
			
			idx = m.adj_matrix[i][j];
			
			if ((idx != 0) &&
			    (idx != (i+1)) &&
			    (isPresent[idx-1] == false)) {
				    
			    	isPresent[idx-1] = true;
			    	adj_list[i].push_back(idx);
			}				
		}
			
		for (k = 0; k < this->size; k++) {
			
			isPresent[k] = false;
		}		
	}
		
	delete[] isPresent;	
}	
	
AdjacencyList::~AdjacencyList() {
	
	if (!isCons) {
		
		delete[] idxArray;
	}
		
	delete[] adj_list;	
}

void AdjacencyList::WriteCons() {
	
	int i;
	std::list<Idx>::iterator it;
	
	for (i = 0; i < this->size; i++) {
	
		std::cout << i+1 << " -> ";
		
		for (it = adj_list[i].begin(); it != adj_list[i].end(); ++it) {

			std::cout << *it << " ";		
		}
			
		std::cout << std::endl;		
	}
}

void AdjacencyList::WriteNonCons() {

	int i;
	std::list<Idx>::iterator it;
	
	for (i = 0; i < this->size; i++) {
	
		std::cout << idxArray[i] << " -> ";
		
		for (it = adj_list[i].begin(); it != adj_list[i].end(); ++it) {
			std::cout << idxArray[*it - 1] << " ";	
		}
			
		std::cout << std::endl;	
	}
}

void AdjacencyList::Write() {

	if (isCons) {
	
		this->WriteCons();
	}
	else {
	
		this->WriteNonCons();
	}
}

bool AdjacencyList::HasCycleRec(int i, bool visitedArray[], bool recArray[]) {

	if (visitedArray[i] == false) {
	
		visitedArray[i] = true;
		recArray[i] = true;
		
		std::list<Idx>::iterator it;
		
		for (it = adj_list[i].begin(); it != adj_list[i].end(); ++it) {
		
			if (!visitedArray[(*it-1)] && HasCycleRec((*it-1), visitedArray, recArray)) {
			
				return true;			
			}
			
			else if (recArray[(*it-1)]) {
			
				return true;			
			}
		}
	}
	
	recArray[i] = false;
	return false;
}

bool AdjacencyList::IsAcyclic() {

	bool * visitedArray = new bool[this->size];
	bool * recArray = new bool[this->size];
	int i;
	std::list<Idx>::iterator it;
	
	for (i = 0; i < this->size; i++) {
	
		visitedArray[i] = false;
		recArray[i] = false;
	}	
	
	for (i = 0; i < this->size; i++) {	
	
		if (HasCycleRec(i, visitedArray, recArray)) {
		
			delete[] visitedArray;
			delete[] recArray;
		
			return false;
		}
	}
	
	delete[] visitedArray;
	delete[] recArray;
	
	return true;
}

int AdjacencyList::FindIdx(Idx index) {

	int i;

	for (i = 0; i < this-> size; i++) {
	
		if (this->idxArray[i] == index) {
		
			return i;	
		}		
	}

	return -1;
}
}
