#pragma once
#include <iostream>
#include <list>
#include "pwt_adjlistud.h"

namespace faudes {

AdjacencyListUndirected::AdjacencyListUndirected(Generator g) {
	
	this->size = g.Size();
	this->adj_list = new std::list<Idx>[size];
	StateSet::Iterator it;
	StateSet::Iterator succ_it;
	StateSet successors;
	int i = 0;
		
	if (IsConsecutive(g)) {
		
		this->isCons = true;
	}
	else {
		
		this->isCons = false;
	}
		
	if (isCons) {
			
		for (it = g.StatesBegin(); it != g.StatesEnd(); ++it) {
		
			adj_list[i].push_back(*it);
			successors = g.SuccessorStates(*it);
			
			for (succ_it = successors.Begin(); succ_it != successors.End(); ++succ_it) {
	
				if (*succ_it != *it) {
				
					adj_list[i].push_back(*succ_it);
					adj_list[*succ_it-1].push_back(i+1);
				}				
			}	
		
			i++;
		}
	}
		
	else {
		int j;
		this->idxArray = new Idx[size];
		
		for (it = g.StatesBegin(); it != g.StatesEnd(); ++it) {
	
			this->idxArray[i] = *it;
			i++;		
		}
			
		i = 0;
			
		for (it = g.StatesBegin(); it != g.StatesEnd(); ++it) {
		
			adj_list[i].push_back(i+1);
			successors = g.SuccessorStates(*it);
				
			for (succ_it = successors.Begin(); succ_it != successors.End(); ++succ_it) {
				
				if (*succ_it != *it) {
					
					j = FindIdx(*succ_it);
				
					adj_list[i].push_back(j+1);
					adj_list[j].push_back(i+1);
				}						
			}
				
			i++;	
		}	
	}
}

AdjacencyListUndirected::AdjacencyListUndirected(Generator g, Idx ev1, Idx ev2) {

	this->size = g.Size();
	this->adj_list = new std::list<Idx>[size];
	const TransSet& trans = g.TransRel();
	StateSet::Iterator st_it;
	TransSet::Iterator trans_it;
	Idx x2;
	int i = 0;
		
	if (IsConsecutive(g)) {
		
		this->isCons = true;
	}
	else {
		
		this->isCons = false;
	}

	if (isCons) {		
		
		for (st_it = g.StatesBegin(); st_it != g.StatesEnd(); st_it++) {
		
			adj_list[i].push_back(*st_it);
				
			if (g.ExistsTransition(*st_it, ev1)) {

				trans_it = g.TransRelBegin(*st_it, ev1);
				x2 = (*trans_it).X2;
							
				if (x2 != *st_it) {
					
					adj_list[i].push_back(x2);
					adj_list[x2-1].push_back(*st_it);
				}
			}

			if (g.ExistsTransition(*st_it, ev2)) {

				trans_it = g.TransRelBegin(*st_it, ev2);
				x2 = (*trans_it).X2;
					
				if (x2 != *st_it) {	
					
					adj_list[i].push_back(x2);
					adj_list[x2-1].push_back(*st_it);
				}
			}

			i++;
		}		
	}
	else {
		int j;
		this->idxArray = new Idx[size];
		
		for (st_it = g.StatesBegin(); st_it != g.StatesEnd(); st_it++) {
			this->idxArray[i] = *st_it;
			i++;		
		}
			
		i = 0;

		for (st_it = g.StatesBegin(); st_it != g.StatesEnd(); st_it++) {

			adj_list[i].push_back(i+1);
				
			if (g.ExistsTransition(*st_it, ev1)) {
				
				trans_it = g.TransRelBegin(*st_it, ev1);
				x2 = (*trans_it).X2;
				j = this->FindIdx(x2);
					
				if (j != i) {
					
					adj_list[i].push_back(j+1);
					adj_list[j].push_back(i+1);
				}
			}

			if (g.ExistsTransition(*st_it, ev2)) {
				
				trans_it = g.TransRelBegin(*st_it, ev2);
				x2 = (*trans_it).X2;
				j = this->FindIdx(x2);
					
				if (j != i) {
					
					adj_list[i].push_back(j+1);
					adj_list[j].push_back(i+1);
				}
			}
		
			i++;
		}	
	}
}	

AdjacencyListUndirected::~AdjacencyListUndirected() {
	
	if (!isCons) {

		delete [] idxArray;
	}
	
	delete[] adj_list;	
}
	
void AdjacencyListUndirected::WriteCons() {

	int i;
	std::list<Idx>::iterator it;
	
	for (i = 0; i < this->size; i++) {
	
		std::cout << i+1 << " -> ";
		
		for (it = adj_list[i].begin(); it != adj_list[i].end(); ++it) {
			std::cout << *it << " ";		
		}
			
		std::cout << std::endl;		
	}
}

void AdjacencyListUndirected::WriteNonCons() {

	int i;
	std::list<Idx>::iterator it;
	
	for (i = 0; i < this->size; i++) {
	
		std::cout << idxArray[i] << " -> ";
		
		for (it = adj_list[i].begin(); it != adj_list[i].end(); ++it) {
			std::cout << idxArray[*it - 1] << " ";	
		}
			
		std::cout << std::endl;	
	}
}

void AdjacencyListUndirected::Write() {

	if (isCons) {
	
		this->WriteCons();
	}
	else {
	
		this->WriteNonCons();
	}
}

int AdjacencyListUndirected::FindIdx(Idx index) {

	int i;

	for (i = 0; i < this-> size; i++) {
	
		if (this->idxArray[i] == index) {
		
			return i;	
		}		
	}

	return -1;
}

bool * AdjacencyListUndirected::ConnectedComponentRec(Idx index, bool * visitedArray) {

	int i = index - 1;
		
	visitedArray[i] = true;	
		
	std::list<Idx>::iterator it;
	
	for (it = adj_list[i].begin(); it != adj_list[i].end(); ++it) {
			
		if (visitedArray[*it-1] == false) {							
			visitedArray = ConnectedComponentRec(*it, visitedArray);
		}
	}
	
	return visitedArray;
}

StateSet AdjacencyListUndirected::ConnectedComponent(Idx index) {

	StateSet states;
	int i = 0;
	
	bool * visitedArray = new bool[this->size];
	
	for (i = 0; i < this->size; i++) {
	
		visitedArray[i] = false;	
	}
	
	visitedArray = this->ConnectedComponentRec(index, visitedArray);

	if (isCons) {
	
		for (i = 0; i < this->size; i++) {
		
			if (visitedArray[i] == true) {
			
				states.Insert(i+1);			
			}		
		}
	}
	
	else {
		Idx j;
	
		for (i = 0; i < this->size; i++) {
		
			if (visitedArray[i] == true) {
			
				j = this->idxArray[i];
				states.Insert(j);			
			}		
		}
	}
	
	delete[] visitedArray;
	return states;
}

StateSet AdjacencyListUndirected::MakeCC(Idx index) {

	if (isCons) {
	
		return ConnectedComponent(index);
	}
	
	else {
	
		int i = FindIdx(index);
		return ConnectedComponent(i+1);	
	}
}

void AdjacencyListUndirected::WriteCC(Idx index) {

	StateSet cc = this->MakeCC(index);
	StateSet::Iterator it;
	
	std::cout << index << " -> ";
	
	for (it = cc.Begin(); it != cc.End(); it++) {
	
		std::cout << *it << " ";
	}

	std::cout << std::endl;
}
}
