#pragma once
#include <iostream>
#include <list>
#include "pwt_adjmatrix.h"

namespace faudes {

AdjacencyMatrix::AdjacencyMatrix(Generator g) {
	
	const TransSet& trans = g.TransRel();
	StateSet successors;
	this->height = g.Size();
	this->width = g.AlphabetSize();
	int i, j;
	this->adj_matrix = new Idx * [this->height];
		
	for (i = 0; i < this->height; i++) {
		
		this->adj_matrix[i] = new Idx[this->width];
	}
				
	StateSet::Iterator succ_it;
	StateSet::Iterator st_it = g.StatesBegin();
	EventSet::Iterator ev_it = g.AlphabetBegin();
		
	if (IsConsecutive(g)) {
		
		this->isCons = true;
	}
	else {
		
		this->isCons = false;
	}
		
	if (isCons) {
		
		for (i = 0; i < this->height; i++) {
		
			for (j = 0; j < this->width; j++)  {
			
				successors = trans.SuccessorStates(*st_it, *ev_it);
				if (successors.Empty()) {
				
					this->adj_matrix[i][j] = 0;
				}
				else {				
					succ_it = successors.Begin();
					this->adj_matrix[i][j] = *succ_it;
				}
				
				ev_it++;
			}
			
			ev_it = g.AlphabetBegin();
			st_it++;
		}	
	}
	else {	
		this->state_array = new Idx[this->height];
		st_it = g.StatesBegin();
			
		for (i = 0; i < this->height; i++) {
			
			this->state_array[i] = *st_it;
			st_it++;			
		}
			
		st_it = g.StatesBegin();
			
		for (i = 0; i < this->height; i++) {
		
			for (j = 0; j < this->width; j++)  {
			
				successors = trans.SuccessorStates(*st_it, *ev_it);
				if (successors.Empty()) {
				
					this->adj_matrix[i][j] = 0;	
				}
				else {
					succ_it = successors.Begin();
					this->adj_matrix[i][j] = FindIdx(*succ_it)+1;	
				}
				
				ev_it++;
			}
			
			ev_it = g.AlphabetBegin();
			st_it++;
		}		
	}					
}

void AdjacencyMatrix::WriteCons() {

	int i, j;
	
	for (i = 0; i < this->height; i++) {
	
		for (j = 0; j < this->width; j++) {
		
			std::cout << this->adj_matrix[i][j] << " ";	
		}
		
		std::cout << std::endl;
	} 
}

void AdjacencyMatrix::WriteNonCons() {

	int i, j, k;
	Idx index;
	
	for (i = 0; i < this->height; i++) {
	
		for (j = 0; j < this->width; j++) {
		
			k = this->adj_matrix[i][j];
			index = this->state_array[k-1];
		
			std::cout << index << " ";	
		}
		
		std::cout << std::endl;
	}	
}

void AdjacencyMatrix::Write() {

	if (isCons) {
	
		this->WriteCons();
	}
	else {
	
		this->WriteNonCons();
	}
}

int AdjacencyMatrix::FindIdx(Idx index) {

	int i;
	
	for (i = 0; i < this->height; i++) {
	
		if (this->state_array[i] == index) {
		
			return i;		
		}	
	}

	return -1;
}
}
