/** @file pwt_include.h 

Include all headers from the piecewise testability plugin.*/

/*
 **************************************************** 
 Convenience header file that includes all headers 
 relevant to the piecewise testability plugin.
 The make system will append an include directive 
 for this file to the "allplugins.h" include file.

 (c) Eliška Foltasová 2021
 ****************************************************
 */

#ifndef FAUDES_PWT_INCLUDE_H
#define FAUDES_PWT_INCLUDE_H

#include "pwt_checkinput.h"
#include "pwt_isacyc.h"
#include "pwt_adjlist.h"
#include "pwt_adjlistud.h"
#include "pwt_adjmatrix.h"
#include "pwt_trahtman.h"
#include "pwt_klimapolak.h"
#include "pwt_klimakuncpolak.h"

#endif

/**

@defgroup PwtestabilityPlugin

@ingroup AllPlugins PwtestabilityPlugin

@section Overview

This plug-in verifies whether a formal language represented by a generator is piecewise testable.
<br>
According to the findings presented by I.Simon in [1], a language is piecewise testable iff its syntactic monoid is J-trivial.
<br>
Following the publication of Simon's article, different authors have created equivalent algorithms verifying this property. The purpose of this plug-in is to provide functions implementing three of these algorithms:


<ul>
<li>Trahtman's algorithm presented in [2]</li>
<li>Klíma and Polák's algorithm presented in [3]</li>
<li>Klíma, Kunc and Polák's algorithm presented in [4]</li>
</ul>

<br>

[1] "Piecewise Testable Events" by I. Simon, Automata Theory and Formal Languages, part of the Lecture Notes in Computer Science book series (vol.33), 214-222, Apr 1975 <br>

[2] "Piecewise and Local Threshold Testability of DFA" by A.N.Trahtman, International Symposium on Fundamentals of Computation Theory, part of the Lecture Notes in Computer Science book series (vol.2138), 347-358, Aug 2001 <br>

[3] "Alternative Automata Characterization of Piecewise Testable Languages" by O. Klíma, M. Kunc and L. Polák, Developments in Language Theory, part of the Lecture Notes in Computer Science book series (vol.7907), 289-300, Jun 2013 <br>

[4] "Deciding k-piecewise Testability" by O.Klíma and L.Polák, submitted manuscript, 2014

@section License


The piecewise testability plugin was developed as a part of E. Foltasová's
bachelor's thesis at the Palacky University in Olomouc. It is distributed with libFAUDES and under the terms of the LGPL, without any warranty. In other words, proceed at your own risk. 

<br>
<br>

Copyright (c) 2021 Eliška Foltasová

@section Contents
@brief

*/
