#pragma once
#include <iostream>
#include <list>
#include "pwt_isacyc.h"

namespace faudes {

bool IsConsecutive(Generator g) {

	StateSet states = g.States();
	int max = states.MaxIndex();	
	int size = g.Size();
	if (size == max) {

		return true;
	}

	return false;
}

Idx * MakeIndexArray(Generator g) {

	int size = g.Size();
	int i = 0;
	StateSet::Iterator it;
	Idx * indexArray = new Idx[size];
	
	for (it = g.StatesBegin(); it != g.StatesEnd(); it++) {
	
		indexArray[i] = *it;
		i++;
	}

	return indexArray;
}

int FindIndex(Generator g, Idx * indexArray, Idx index) {

	int i;
	for (i = 0; i < g.Size(); i++) {
		if (indexArray[i] == index) {

			return i;
		}
	}

	return -1;
}

bool HasCycleNonCons(Generator g, int i, bool visitedArray[], bool recArray[], Idx indexArray[]) 
{ 
	int j;
    	if (visitedArray[i] == false) {
    	 
        	visitedArray[i] = true; 
        	recArray[i] = true; 
  
       	StateSet::Iterator it;
		StateSet succ = g.SuccessorStates(indexArray[i]);
		succ.Erase(indexArray[i]);
		
        	for (it = succ.Begin(); it != succ.End(); it++) {
			
			j = FindIndex(g, indexArray, *it);
			
            		if (!visitedArray[j] && HasCycleNonCons(g, j, visitedArray, recArray, indexArray)) {
            		                	
				return true; 
			}
            		else if (recArray[j]) {
	
        	        	return true; 
			}
        	} 
  
    	} 

    	recArray[i] = false; 
    	return false; 
} 
  
bool IsAcyclicNonCons(Generator g) 
{ 
	int i;
	int size = g.Size();
	Idx * indexArray = MakeIndexArray(g);
	bool * visitedArray = new bool[size]; 
	bool * recArray = new bool[size]; 

	for (i = 0; i < size; i++) 
	{ 
		visitedArray[i] = false; 
        	recArray[i] = false; 
    	} 

	StateSet::Iterator it;

    	for (it = g.StatesBegin(); it != g.StatesEnd(); it++) {
		
        	i = FindIndex(g, indexArray, *it);
		if (HasCycleNonCons(g, i, visitedArray, recArray, indexArray))  {
			delete[] visitedArray;
			delete[] recArray;
			delete[] indexArray;
	
			return false;
		}
	}
	
	delete[] visitedArray;
	delete[] recArray;
	delete[] indexArray;
  
  	return true; 
}

bool HasCycleCons(Generator g, int i, bool visitedArray[], bool recArray[]) 
{ 
	if (visitedArray[i-1] == false) { 
        	
        	visitedArray[i-1] = true; 
        	recArray[i-1] = true; 
  
        	StateSet::Iterator it;
		StateSet succ = g.SuccessorStates(i);
		succ.Erase(i);
		 
        	for (it = succ.Begin(); it != succ.End(); it++) {
        	 
			if (!visitedArray[(*it-1)] && HasCycleCons(g, *it, visitedArray, recArray)) {

				return true;
			} 
            		else {
				if (recArray[(*it-1)]) {
				
                			return true;
				}
			}
        	}  
   	}

    	recArray[i-1] = false; 
    	return false; 
} 
  
bool IsAcyclicCons(Generator g) 
{ 
    	int i;
    	int size = g.Size();
    	bool *visitedArray = new bool[size]; 
    	bool *recArray = new bool[size];
    	 
   	for (i = 0; i < size; i++) 
    	{ 
        	visitedArray[i] = false; 
        	recArray[i] = false; 
    	} 

	StateSet::Iterator it;

	for (it = g.StatesBegin(); it != g.StatesEnd(); it++) {

        	if (HasCycleCons(g, *it, visitedArray, recArray))  {
	
			delete[] visitedArray;
			delete[] recArray;
			
			return false;
		}
	}
	
	delete[] visitedArray;
	delete[] recArray;
  
	return true; 
}

bool IsAcyclic(Generator g)
{
	if (IsConsecutive(g)) {
	
		return IsAcyclicCons(g);
	}
	else {	
		return IsAcyclicNonCons(g);
	}
}
}
