#pragma once
#include <iostream>
#include <list>
#include "pwt_klimakuncpolak.h"

namespace faudes {

bool IsBAbsorbing(Generator g, Idx st, Idx ev1, Idx ev2) {

	if (g.ExistsTransition(st, ev1, st) &&
	    g.ExistsTransition(st, ev2, st)) {
	    
		return true;
	}
	
	return false;
}

bool HasOneBAbsSt(Generator g, StateSet set, Idx ev1, Idx ev2) {

	int size = set.Size();	
	StateSet::Iterator it;
	int count = 0;
	
	for (it = set.Begin(); it != set.End(); it++) {

		if (IsBAbsorbing(g, *it, ev1, ev2)) {

			count++;
		}
		
		if (count > 1) {			

			return false;
		}				
	}

	return true;
}

std::list<StateSet> MakeStateSetList(Generator g, Idx ev1, Idx ev2) {

	std::list<StateSet> st_list;
	AdjacencyListUndirected adj(g, ev1, ev2);
	StateSet::Iterator st_it1;
	StateSet::Iterator st_it2;
	int size = g.Size();
	int i, j, k;
	StateSet states;
	bool * hasBeenVisited = new bool[size];
	
	for (i = 0; i < size; i++) {
		
		hasBeenVisited[i] = false;
	}

	if (IsConsecutive(g)) {

		for (st_it1 = g.StatesBegin(); st_it1 != g.StatesEnd(); st_it1++) {
			
			if (hasBeenVisited[(*st_it1) - 1] == false) {
			
				states = adj.MakeCC(*st_it1);
				st_list.push_back(states);

				for (st_it2 = states.Begin(); st_it2 != states.End(); st_it2++) {
					
					hasBeenVisited[(*st_it2) - 1] = true;
				}				
			}
		}
	}
	else {
		for (st_it1 = g.StatesBegin(); st_it1 != g.StatesEnd(); st_it1++) {
			
			j = adj.FindIdx(*st_it1);
			
			if (hasBeenVisited[j] == false) {

				states = adj.MakeCC(*st_it1);		
				st_list.push_back(states);

				for (st_it2 = states.Begin(); st_it2 != states.End(); st_it2++) {
					
					k = adj.FindIdx(*st_it2);
					hasBeenVisited[k] = true;
				}			
			}
		}
	}

	delete[] hasBeenVisited;

	return st_list;
}

void WriteStateSetList(Generator g, Idx ev1, Idx ev2) {

	std::list<StateSet> st_list = MakeStateSetList(g, ev1, ev2);
	StateSet::Iterator st_it;
	StateSet states;
	std::list<StateSet>::iterator list_it;
	int i;
	
	for (list_it = st_list.begin(); list_it != st_list.end(); ++list_it) {
	
		states = *list_it;	
			
		for (st_it = states.Begin(); st_it != states.End(); st_it++) {
		
			std::cout << *st_it << " ";		
		}
		
		std::cout << std::endl;	
	}
}

bool IsPiecewiseTestable_KlimaKuncPolak(Generator g) {

	g = CheckInputProps(g);

	AdjacencyList adj(g);

	if (adj.IsAcyclic() == false) {
	
		return false;
	}
	
	EventSet::Iterator ev_it1;
	EventSet::Iterator ev_it2;
	std::list<StateSet> set;
	std::list<StateSet>::iterator it;

	for (ev_it1 = g.AlphabetBegin(); ev_it1 != g.AlphabetEnd(); ev_it1++) {
		for (ev_it2 = g.AlphabetBegin(); ev_it2 != g.AlphabetEnd(); ev_it2++) {
		
			if (*ev_it1 != *ev_it2) {
			
				set = MakeStateSetList(g, *ev_it1, *ev_it2);
			
				for (it = set.begin(); it != set.end(); ++it) {
	
					if (HasOneBAbsSt(g, *it, *ev_it1, *ev_it2) == false) {
				
					return false;
					}
				}
			}	
		}	
	}
	
	return true;
}
}
