#pragma once
#include <iostream>
#include <list>
#include "pwt_checkinput.h"
#include "pwt_trahtman.h"

namespace faudes {

bool IsLoop(Transition t) {

	if (t.X1 == t.X2) {

		return true;
	}

	return false;
}

bool NoOutgoingEdges(Generator g, Idx i) {

	const TransSet& stateTransSet = g.ActiveTransSet(i);
	
	if (stateTransSet.Size() == 0) {
	
		return true;
	}
	
	TransSet::Iterator it;

	for (it = stateTransSet.Begin(); it != stateTransSet.End(); it++) {

		if (!IsLoop(*it)) {

			return false;
		}
	}

	return true;
}

EventSet MakeStabilizer(Generator g, Idx i) {

	EventSet stabilizer = EventSet();
	const TransSet& trans = g.ActiveTransSet(i);
	TransSet::Iterator it;
	
	for (it = trans.Begin(); it != trans.End(); it++) {

		if (IsLoop(*it)) {

			stabilizer.Insert((*it).Ev);
		}
	}

	return stabilizer;
}

Generator MakeStabilizerGraph(Generator g, Idx i) {

	EventSet e = MakeStabilizer(g, i);
	Generator newGen(g);
	newGen.RestrictAlphabet(e);
	return newGen;
}

bool IsPiecewiseTestable_Trahtman(Generator g) {

	g = CheckInputProps(g);

	if (IsAcyclic(g) == false) {

		return false;
	}

	StateSet states = g.States();
	StateSet conComp;
	StateSet::Iterator it1;
	StateSet::Iterator it2;

	for (it1 = states.Begin(); it1 != states.End(); ++it1) {

		Generator stabilizerGraph = MakeStabilizerGraph(g, *it1);
		AdjacencyListUndirected undirectedCopy(stabilizerGraph);
		conComp = undirectedCopy.MakeCC(*it1);

		for (it2 = conComp.Begin(); it2 != conComp.End(); ++it2) {

			if ((*it2) != (*it1)) {

				if (NoOutgoingEdges(stabilizerGraph, (*it2))) {
				
					return false;
				}
			}
		}
	}

	return true;
}
}
