/** @file pwt_tutorial_klimakuncpolak.cpp  

Tutorial, piecewise testability plugin. This tutorial demonstrates the use of Klíma, Kunc and Polák's algorithm.

@ingroup Tutorials 

@include pwt_tutorial_klimakuncpolak.cpp

*/

#include <iostream>
#include <list>
#include "libfaudes.h"

using namespace faudes;

int main() {

	Generator g1;
	
	std::cout << "Loading generator g1..." << std::endl;
	
	g1.Read("data/g4.gen");
	
	g1.Write();
	
	std::cout << "---------------------------" << std::endl;
	
	std::cout << "Undirected copy of g1 represented by an adjacency list:" << std::endl;
		
	AdjacencyListUndirected lu1(g1);
	
	lu1.Write();
	
	std::cout << "---------------------------" << std::endl;
	
	std::cout << "Here's a weakly connected component of g1 containing a state indexed by 20:" << std::endl;
	
	lu1.WriteCC(20);
	
	std::cout << "---------------------------" << std::endl;
	
	AdjacencyList l1(g1);
	
	bool b = l1.IsAcyclic();
	
	if (b == true) {
	
	std::cout << "Generator g1 is acyclic. Let's see if its accepted language is piecewise testable." << std::endl;
		
		bool c = IsPiecewiseTestable_KlimaKuncPolak(g1);
		
		if (c == true) {
		
			std::cout << "Generator g1 accepts a language which is piecewise testable." << std::endl;
		}
		else {
		
			std::cout << "Generator g1 accepts a language which is not piecewise testable." << std::endl;
		}
	}
	else {
	
		std::cout << "Generator g1 is not acyclic, therefore its accepted language cannot be piecewise testable." << std::endl;
	}
	
	std::cout << "---------------------------" << std::endl;
	
	Generator g2;
	
	std::cout << "Loading generator g2..." << std::endl;
	
	g2.Read("data/g2.gen");
	
	g2.Write();
	
	std::cout << "---------------------------" << std::endl;
	
	std::cout << "Generator g2 represented by an undirected adjacency list:" << std::endl;
		
	AdjacencyListUndirected lu2(g2);
	
	lu2.Write();
	
	std::cout << "---------------------------" << std::endl;
	
	std::cout << "Here's a weakly connected component of g2 containing a state indexed by 3:" << std::endl;
	
	lu2.WriteCC(3);
	
	std::cout << "---------------------------" << std::endl;
	
	AdjacencyList l2(g2);
	
	b = l2.IsAcyclic();
	
	if (b == true) {
	
		std::cout << "Generator g2 is acyclic. Let's see if its accepted language is piecewise testable." << std::endl;
		
		bool d = IsPiecewiseTestable_KlimaKuncPolak(g2);
		
		if (d == true) {
		
			std::cout << "Generator g2 accepts a language which is piecewise testable." << std::endl;
		}
		else {
		
			std::cout << "Generator g2 accepts a language which is not piecewise testable." << std::endl;
		}
	}
	else {
	
		std::cout << "Generator g2 is not acyclic, therefore its accepted language cannot be piecewise testable." << std::endl;
	}
	
	return 0;
}
