/** @file pwt_tutorial_klimapolak.cpp  

Tutorial, piecewise testability plugin. This tutorial demonstrates the use of Klíma and Polák's algorithm implementation.

@ingroup Tutorials 

@include pwt_tutorial_klimapolak.cpp

*/

#include <iostream>
#include <list>
#include "libfaudes.h"

using namespace faudes;

int main() {

	Generator g1;
	
	std::cout << "Loading generator g1..." << std::endl;
	
	g1.Read("data/g5.gen");
	
	g1.Write();
	
	std::cout << "Generator g1 represented by an adjacency matrix:" << std::endl;
	
	AdjacencyMatrix m1(g1);
	
	m1.Write();
	
	std::cout << "---------------------------" << std::endl;
	
	std::cout << "Generator g1 represented by an adjacency list:" << std::endl;
	
	AdjacencyList l1(m1);
	
	l1.Write();
	
	std::cout << "---------------------------" << std::endl;
	
	bool b = l1.IsAcyclic();
	
	if (b == true) {
	
		std::cout << "Generator g1 is acyclic. Let's see if its accepted language is piecewise testable." << std::endl;
		
		bool c = IsPiecewiseTestable_KlimaPolak(g1);
		
		if (c == true) {
		
			std::cout << "Generator g1 accepts a language which is piecewise testable." << std::endl;
		}
		else {
		
			std::cout << "Generator g1 accepts a language which is not piecewise testable." << std::endl;
		}
	}
	else {
	
		std::cout << "Generator g1 is not acyclic, therefore its accepted language cannot be piecewise testable." << std::endl;
	}
	
	std::cout << "---------------------------" << std::endl;
	
	Generator g2;
	
	std::cout << "Loading generator g2..." << std::endl;
	
	g2.Read("data/g3.gen");
	
	g2.Write();
	
	std::cout << "Generator g2 represented by an adjacency matrix:" << std::endl;
	
	AdjacencyMatrix m2(g2);
	
	m2.Write();
	
	std::cout << "---------------------------" << std::endl;
	
	std::cout << "Generator g2 represented by an adjacency list:" << std::endl;
	
	AdjacencyList l2(m2);
	
	l2.Write();
	
	std::cout << "---------------------------" << std::endl;
	
	b = l2.IsAcyclic();
	
	if (b == true) {
	
		std::cout << "Generator g2 is acyclic. Let's see if its accepted language is piecewise testable." << std::endl;
		
		bool d = IsPiecewiseTestable_KlimaPolak(g2);
		
		if (d == true) {
		
			std::cout << "Generator g2 accepts a language which is piecewise testable." << std::endl;
		}
		else {
		
			std::cout << "Generator g2 accepts a language which is not piecewise testable." << std::endl;
		}
	}
	else {
	
		std::cout << "Generator g2 is not acyclic, therefore its accepted language cannot be piecewise testable." << std::endl;
	}
	
	return 0;
}
