/** @file pwt_tutorial_trahtman.cpp  

Tutorial, piecewise testability plugin. This tutorial demonstrates
the use of Trahtman's algorithm implementation.

@ingroup Tutorials 

@include pwt_tutorial_trahtman.cpp

*/

#include <iostream>
#include <list>
#include "libfaudes.h"

using namespace faudes;

int main() {

	Generator g1, g2;
	
	std::cout << "Loading generator g1..." << std::endl;
	
	g1.Read("data/g1.gen");
	
	g1.Write();
	
	bool b = IsAcyclic(g1);
	
	if (b == true) {
	
		std::cout << "Generator g1 is acyclic. Let's see if its accepted language is piecewise testable." << std::endl;
		
		bool c = IsPiecewiseTestable_Trahtman(g1);
		
		if (c == true) {
		
			std::cout << "Generator g1 accepts a language which is piecewise testable." << std::endl;
		}
		else {
		
			std::cout << "Generator g1 accepts a language which is not piecewise testable." << std::endl;
		}
	}
	else {
	
		std::cout << "Generator g1 is not acyclic, therefore its accepted language cannot be piecewise testable." << std::endl;
	}
	
	std::cout << "---------------------------" << std::endl;
	std::cout << "Loading generator g2..." << std::endl;
	
	g2.Read("data/g6.gen");
	
	g2.Write();
	
	b = IsAcyclic(g2);
	
	if (b == true) {
	
		std::cout << "Generator g2 is acyclic. Let's see if its accepted language is piecewise testable." << std::endl;
		
		bool d = IsPiecewiseTestable_Trahtman(g2);
		
		if (d == true) {
		
			std::cout << "Generator g2 accepts a language which is piecewise testable." << std::endl;
		}
		else {
		
			std::cout << "Generator g2 accepts a language which is not piecewise testable." << std::endl;
		}
	}
	else {
	
		std::cout << "Generator g2 is not acyclic, therefore its accepted language cannot be piecewise testable." << std::endl;
	}
	
	std::cout << "---------------------------" << std::endl;
	
	std::cout << "Here's how a stabilizer (a set of events represented by indices) looks for generator g1 and its state indexed by 2:" << std::endl;
	
	EventSet stab = MakeStabilizer(g1, 2);
	
	EventSet::Iterator it;
	
	for (it = stab.Begin(); it != stab.End(); it++) {
	
		std::cout << *it << " ";
	}
	
	std::cout << std::endl;
	
	std::cout << "---------------------------" << std::endl;
	
	std::cout << "Here's how a stabilizer graph looks like for the same generator and the same state:" << std::endl;
	
	Generator g3 = MakeStabilizerGraph(g1, 2);
	
	g3.Write();

	return 0;
}
